/*	lock.c		4/28/92
 *
 * Copyright 1992-1996  Perry R. Ross
 *
 *  This program is free software; you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License  
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "ldb.h"


/*----------------------------------------------------------------------
 *	get_lock -- attempt to open a lock file, exit if it exists
 *
 * This function attempts to create a file in such a way that,
 * if the file exists, the create will fail.  If the create succeeds,
 * we "own" the ldb lock, and have the exclusive right to access
 * the ldb data files in this directory.  If the create fails,
 * some other ldb already has the lock and we exit.
 *
 * This uses the O_EXCL flag to open, which as far as I know is
 * supported on all UNIX flavors.  If it isn't on yours:
 *	a. Please mail me at the address in patchlevel.h for AUTHOR_EMAIL
 *	   and tell me what your system is.
 *	b. try replacing open(fn,O_...,0644) with creat(fn,0444).
 *----------------------------------------------------------------------
 */

void
get_lock(fn)
char *fn;
{
int fd;
struct stat s;

if ( (fd = open(fn,O_WRONLY|O_CREAT|O_EXCL,0644)) < 0) {
	printf("ERROR: Ldb is already running in this account -- exiting.\n\n");
	printf("If you are sure no other ldb is active, you can remove\n");
	printf("the lock with the following command:\n\n");
#ifdef VMS
	printf("\t$ delete sys$login:%s;*\n",fn);
#else
	if (*fn == '/')
		printf("\t$ rm %s\n",fn);
	else
		printf("\t$ rm $HOME/%s\n",fn);
#endif
	printf(
	"\nPlease be certain no other ldb is running before doing this.\n\n");
	if (stat(fn,&s) >= 0)
		printf("The lock file was created %s\n",ctime(&s.st_ctime));
	exit(STAT_ABORT);	/* DON'T call ldbexit, it calls release_lock */
	}
close(fd);		/* success, close the file and return */
}
